/* Population.hpp
   interface for the class population. It is implemented as a pointer to an 
   array of chromosomes
   @author	Hyun Kim
   @date	11/20/99
*/

#ifndef _POPULATION_HPP
#define _POPULATION_HPP

#include "Chromosome.hpp"
#include <iostream.h>
#include <fstream.h>

class Population
{
	friend ostream& operator<<( ostream& out, Population& pop );
	friend ofstream& operator<<( ofstream& out, Population& pop );

private:
	Chromosome*	chromosomes;	// pointer to an array of chromosomes
	Chromosome	child1, child2;
	int			size;			// size of the population
	double		sumFitness, maxFitness, minFitness, avgFitness;
	int			maxIndex, minIndex;

	int			getIndexAt( int n );	// return the index which has the Nth largest fitness
	Chromosome	LINXchild( Chromosome& parent1, Chromosome& parent2 );

	// list of selection operators, called by the method selection()
	int			rouletteWheelSelection();

	// list of crossover operators, called by the method crossover()
	void		doPTX1( Chromosome& parent1, Chromosome& parent2 );
	void		doUNIFORM( Chromosome& parent1, Chromosome& parent2 );
	void		doLINX( Chromosome& parent1, Chromosome& parent2 );

public:
	Population();
	Population( Population& pop );
	~Population();
	Chromosome&	operator[]( int i );	// return the pointer to the i-th chromosome
	Population&	operator=( Population& newPop );
	void		setSize( int n ) { size = n; };
	int			getSize() const	{ return size; };
	double		getSumFitness() const { return sumFitness; };
	double		getMaxFitness() const { return maxFitness; };
	double		getMinFitness() const { return minFitness; };
	double		getAvgFitness() const { return avgFitness; };
	int			getMaxIndex() const { return maxIndex; };
	int			getMinIndex() const { return minIndex; };
	Chromosome	getChild1() { return child1; };
	Chromosome	getChild2() { return child2; };
	Population	reproduce();
	Population	reproduceWithReplacement();
	void		statistics();	// calculate the total/max/min/avg of the population fitnesses
	void		initialize();
	int			selection();
	void		crossover( Chromosome& parent1, Chromosome& parent2 );	// produce children using a crossove operator

};

#endif

